import { Connection } from 'typeorm';
import { InjectConnection } from '@nestjs/typeorm';
import { Injectable, ConsoleLogger } from '@nestjs/common';
import { ImageService } from './image/image.service';
import { AvatarApiService } from './avatar-api/avatar-api.service';
import { Avatar } from './entities/Avatar.entity';

@Injectable()
export class AppService extends ConsoleLogger {
  constructor(
    @InjectConnection()
    private db: Connection,
    private image: ImageService,
    private avatarApi: AvatarApiService,
  ) {
    super('app');
  }

  async getImage(username: string, size: number) {
    const url = await this.avatarApi.getURLFromUsername(username);
    const existingAvatar = await this.db.getRepository(Avatar).findOne({
      where: {
        url,
        size,
      },
    });
    if (existingAvatar) {
      return existingAvatar.toBuffer();
    }
    let originalAvatarBuffer: Buffer;
    const originalAvatar = await this.db.getRepository(Avatar).findOne({
      where: {
        url,
        size: 0,
      },
    });
    if (originalAvatar) {
      originalAvatarBuffer = originalAvatar.toBuffer();
    } else {
      originalAvatarBuffer = await this.avatarApi.downloadAvatar(url);
      try {
        await this.db.getRepository(Avatar).save({
          url,
          size: 0,
          buffer: originalAvatarBuffer,
        });
      } catch (e) {
        this.error(`Fail to save original avatar: ${e.message}`);
      }
    }
    const avatarBuffer = await this.image.resize(originalAvatarBuffer, size);
    try {
      const avatar = new Avatar();
      avatar.url = url;
      avatar.size = size;
      avatar.fromBuffer(avatarBuffer);
      await this.db.getRepository(Avatar).save(avatar);
    } catch (e) {
      this.error(`Fail to save avatar: ${e.message}`);
    }
    return avatarBuffer;
  }
}
